module Types.RoomsState
  ( RoomsState,
    initRoomsState,
    HasRoomsState (..),
    roomStateDiffers,
    updateRoomState,
    getRoomState,
  )
where

import ClassyPrelude
import State.RoomDataState (MonadRoomDataStateRead (getRoomDataState))
import Types.RoomData (RoomsData)

type RoomsState = TVar RoomsData

initRoomsState :: IO RoomsState
initRoomsState = newTVarIO []

class HasRoomsState a where
  getRoomsState :: a -> RoomsState

updateRoomState ::
  ( HasRoomsState env,
    MonadIO m,
    MonadReader env m
  ) =>
  RoomsData ->
  m ()
updateRoomState newData = do
  state <- getRoomsState <$> ask
  _ <- atomically $ swapTVar state newData
  return ()

getRoomState ::
  ( HasRoomsState env,
    MonadIO m,
    MonadReader env m
  ) =>
  m RoomsData
getRoomState = do
  state <- getRoomsState <$> ask
  readTVarIO state

roomStateDiffers ::
  ( MonadRoomDataStateRead m
  ) =>
  RoomsData ->
  m Bool
roomStateDiffers newData = do
  not . eqIgnoreOrdering newData <$> getRoomDataState

eqIgnoreOrdering :: (Eq a) => [a] -> [a] -> Bool
eqIgnoreOrdering a b = length a == length b && all (`elem` b) a
